package caf.war.wm_hpstra.adminportlet;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.model.SelectItem;

import com.webmethods.caf.faces.annotations.BeanType;
import com.webmethods.caf.faces.annotations.DTManagedBean;
import com.webmethods.caf.faces.annotations.ExpireWithPageFlow;
import com.webmethods.caf.faces.bean.BasePageBean;
import com.webmethods.caf.faces.data.ISelectItemGroupProvider;
import com.webmethods.caf.faces.data.object.TableSelectItemGroupProvider;
import com.webmethods.caf.shared.hpstra.DataProviderType;
import com.webmethods.caf.shared.hpstra.DataProviderType.StandardProviderType;
import com.webmethods.caf.shared.hpstra.DataStoreConfig;
import com.webmethods.caf.shared.hpstra.HPSTRAManager;
import com.webmethods.caf.shared.hpstra.db.DBConfig;
import com.webmethods.caf.shared.hpstra.es.ESConfig;
import com.webmethods.portal.bizPolicy.IContext;
import com.webmethods.portal.bizPolicy.command.ICommand;
import com.webmethods.portal.bizPolicy.command.ICommandManager;
import com.webmethods.portal.bizPolicy.command.sql.ListDataSources;
import com.webmethods.portal.bizPolicy.impl.ContextFactory;
import com.webmethods.portal.service.sql.core.ConnectionInfo;
import com.webmethods.portal.system.PortalSystem;

/**
 * Administration portlet, used for configuring HPSTRA. Possible options are - Elastic
 * Search, Data source or a custom class implementation.
 * 
 * @author ivat
 */
@ManagedBean(name = "AdminPortletDefaultviewView")
@SessionScoped
@ExpireWithPageFlow
@DTManagedBean(displayName = "AdminPortlet/default", beanType = BeanType.PAGE)
public class AdminPortletDefaultviewView extends BasePageBean {

	private static final String CUSTOM_DATA_SOURCE = "OTHER"; //$NON-NLS-1$

	private static final String[][] INITIALIZE_PROPERTY_BINDINGS = new String[][] {};

	/**
	 * Determines if a de-serialized file is compatible with this class.
	 *
	 * Maintainers must change this value if and only if the new version of this class is
	 * not compatible with old versions. See Sun docs for <a
	 * href=http://java.sun.com/j2se/1.5.0/docs/guide/serialization/spec/version.html>
	 * details. </a>
	 */
	private static final long serialVersionUID = 1L;

	private transient AdminPortlet adminPortlet;

	private String dbDataSource;

	private ISelectItemGroupProvider dbDatasources;

	private String esHost;

	private String esIndex;

	private String esPassword;

	private int esPort;

	private String esUsername;

	private String implClass;

	private String storageType;

	private ISelectItemGroupProvider storageTypes;

	public AdminPortlet getAdminPortlet() {
		if (this.adminPortlet == null) {
			this.adminPortlet = (AdminPortlet) resolveExpression("#{AdminPortlet}"); //$NON-NLS-1$
		}
		return this.adminPortlet;
	}

	public String getDbDataSource() {
		return this.dbDataSource;
	}

	public ISelectItemGroupProvider getDbDatasources() {
		return this.dbDatasources;
	}

	public String getEsHost() {
		return this.esHost;
	}

	public String getEsIndex() {
		return this.esIndex;
	}

	public String getEsPassword() {
		return this.esPassword;
	}

	public int getEsPort() {
		return this.esPort;
	}

	public String getEsUsername() {
		return this.esUsername;
	}

	public String getImplClass() {
		return this.implClass;
	}

	public String getStorageType() {
		return this.storageType;
	}

	public ISelectItemGroupProvider getStorageTypes() {
		return this.storageTypes;
	}

	/**
	 * Initialize page. Loads the possible configuration choices for HPSTRA data provider
	 * - Elastic Search, Data source or a custom implementation. Fetches the already
	 * configured data sources.
	 */
	@Override
	public String initialize() {
		try {
			resolveDataBinding(INITIALIZE_PROPERTY_BINDINGS, null, "initialize", true, false); //$NON-NLS-1$
			List<SelectItem> list = new ArrayList<>();
			list.add(new SelectItem(StandardProviderType.DATABASE.name(), "Database")); //$NON-NLS-1$
			list.add(new SelectItem(StandardProviderType.ELASTIC_SEARCH.name(), "Elastic Search")); //$NON-NLS-1$
			list.add(new SelectItem(CUSTOM_DATA_SOURCE, "Custom")); //$NON-NLS-1$
			this.storageTypes = new TableSelectItemGroupProvider(list, "label", "value"); //$NON-NLS-1$ //$NON-NLS-2$

			final DataStoreConfig dsc = HPSTRAManager.getInstance().getConfig();
			final DataProviderType dpt = dsc.getDataProvType();
			StandardProviderType configuredType = null;
			for (StandardProviderType spt : StandardProviderType.values()) {
				if (spt.getProviderType() == dpt) {
					configuredType = spt;
					break;
				}
			}
			if (configuredType != null) {
				this.storageType = configuredType.name();
				final Properties props = dsc.getParams();
				switch (configuredType) {
					case DATABASE:
						this.dbDataSource = props.getProperty(DBConfig.Param.DATASOURCE.toString());
						break;
					case ELASTIC_SEARCH:
						this.esHost = props.getProperty(ESConfig.Param.HOST.toString());
						this.esPort = Integer.parseInt(props.getProperty(ESConfig.Param.PORT.toString(),
								Integer.toString(0)));
						this.esIndex = props.getProperty(ESConfig.Param.INDEX.toString());
						this.esUsername = props.getProperty(ESConfig.Param.USERNAME.toString());
						this.esPassword = props.getProperty(ESConfig.Param.PASSWORD.toString());
						break;
					default:
						break;
				}

			} else {
				this.implClass = dpt.toString();
			}

			list = new ArrayList<>();
			final IContext ctx = ContextFactory.acquireContext(true);
			final ICommand command = ((ICommandManager) PortalSystem.getCommandProvider()).getCommand(ctx,
					ListDataSources.COMMAND_NAME);
			for (final Object connInfoObj : (List<?>) command.handle(ctx, null)) {
				final ConnectionInfo connInfo = (ConnectionInfo) connInfoObj;
				list.add(new SelectItem(connInfo.getName(), connInfo.getDisplayName()));
			}
			this.dbDatasources = new TableSelectItemGroupProvider(list, "label", "value"); //$NON-NLS-1$ //$NON-NLS-2$
		} catch (final Exception e) {
			error(e);
			log(e);
		}
		return null;
	}

	/**
	 * Saves the HPSTRA configuraion. Reads the set data provider and its corresponding
	 * parameters and reconfigures the HPSTRA manager.
	 */
	public void saveConfig() {
		try {
			final Properties props = new Properties();
			if (this.storageType.equals(CUSTOM_DATA_SOURCE)) {
				props.setProperty(DataStoreConfig.DATA_PROVIDER_TYPE, this.implClass);
			} else {
				final StandardProviderType spt = StandardProviderType.valueOf(this.storageType);
				props.setProperty(DataStoreConfig.DATA_PROVIDER_TYPE, spt.getProviderType().toString());
				switch (spt) {
					case DATABASE:
						props.setProperty(DBConfig.Param.DATASOURCE.toString(), this.dbDataSource);
						break;
					case ELASTIC_SEARCH:
						props.setProperty(ESConfig.Param.HOST.toString(), this.esHost);
						props.setProperty(ESConfig.Param.PORT.toString(), Integer.toString(this.esPort));
						props.setProperty(ESConfig.Param.INDEX.toString(), this.esIndex);
						props.setProperty(ESConfig.Param.USERNAME.toString(), this.esUsername);
						props.setProperty(ESConfig.Param.PASSWORD.toString(), this.esPassword);
						break;
					default:
						break;
				}
			}
			HPSTRAManager.getInstance().setConfig(new DataStoreConfig(props));
			getFacesContext().addMessage(null, new FacesMessage("Successfully Configured HPSTRA Storage")); //$NON-NLS-1$
		} catch (final Exception e) {
			error(e);
			log(e);
		}
	}

	public void setDbDataSource(final String dbDataSource) {
		this.dbDataSource = dbDataSource;
	}

	public void setEsHost(final String esHost) {
		this.esHost = esHost;
	}

	public void setEsIndex(final String esIndex) {
		this.esIndex = esIndex;
	}

	public void setEsPassword(final String esPassword) {
		this.esPassword = esPassword;
	}

	public void setEsPort(final int esPort) {
		this.esPort = esPort;
	}

	public void setEsUsername(final String esUsername) {
		this.esUsername = esUsername;
	}

	public void setImplClass(String implClass) {
		this.implClass = implClass;
	}

	public void setStorageType(final String storageType) {
		this.storageType = storageType;
	}

}