package caf.war.wm_hpstra.purgetasksportlet;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.model.SelectItem;

import com.webmethods.caf.faces.annotations.BeanType;
import com.webmethods.caf.faces.annotations.DTManagedBean;
import com.webmethods.caf.faces.annotations.ExpireWithPageFlow;
import com.webmethods.caf.faces.bean.BasePageBean;
import com.webmethods.caf.faces.data.ISelectItemGroupProvider;
import com.webmethods.caf.faces.data.object.TableSelectItemGroupProvider;
import com.webmethods.caf.faces.data.task.ITask;
import com.webmethods.caf.faces.data.task.ITaskConstants;
import com.webmethods.caf.faces.data.task.ITaskSearchQuery;
import com.webmethods.caf.faces.data.task.ITaskSearchRecord;
import com.webmethods.caf.faces.data.task.impl.TaskSearchQueryV2;
import com.webmethods.caf.shared.hpstra.HPSTRAManager;
import com.webmethods.portal.PortalException;
import com.webmethods.portal.mech.task.impl.TaskHelper;
import com.webmethods.portal.service.task.ITaskDefinition;

/**
 * A helper portlet which synchronizes tasks types with the HPSTRA storage and deletes
 * tasks from there as well.
 * 
 * @author ivat
 */
@ManagedBean(name = "PurgeTasksPortletDefaultviewView")
@SessionScoped
@ExpireWithPageFlow
@DTManagedBean(displayName = "PurgeTasksPortlet/default", beanType = BeanType.PAGE)
public class PurgeTasksPortletDefaultviewView extends BasePageBean {

	private static final String[][] INITIALIZE_PROPERTY_BINDINGS = new String[][] {};

	private static final long serialVersionUID = 1L;

	private transient PurgeTasksPortlet purgeTasksPortlet;

	private int taskCount;

	private int tasksToDelete;

	private String taskTypeId;

	private ISelectItemGroupProvider taskTypes;

	/**
	 * Deletes tasks from the HPSTRA storage by a given tasks number and a task
	 * definition.
	 * 
	 * @throws Exception
	 *             if any problem occurs
	 */
	public void deleteTasks() throws Exception {
		long time = System.currentTimeMillis();
		final HPSTRAManager mng = HPSTRAManager.getInstance();
		final TaskSearchQueryV2 searchQuery = new TaskSearchQueryV2();
		searchQuery.addQueryTerm(new String[] { ITaskConstants.TASK_FIELD_TASK_TYPE_ID },
				ITaskSearchQuery.OPERATOR_EQUALS, this.taskTypeId);
		searchQuery.setToIndex(this.tasksToDelete);
		final List<ITask> tasks = new ArrayList<>();
		final List<ITaskSearchRecord> records = mng.searchTasks(searchQuery, false);
		for (final ITaskSearchRecord record : records) {
			tasks.add(record.getTask());
		}
		mng.deleteTasks(tasks);
		time = System.currentTimeMillis() - time;
		Thread.sleep(1000);
		fetchCount();
		final String msg = MessageFormat.format(
				"Successfully deleted {0} tasks in {1} seconds from HPSTRA storage", tasks.size(), //$NON-NLS-1$
				time / 1000.0);
		getFacesContext().addMessage(null, new FacesMessage(msg));
	}

	/**
	 * Counts tasks, stored in HPSTRA by a given task definition.
	 * 
	 * @throws PortalException
	 *             if any problem occurs
	 */
	public void fetchCount() throws PortalException {
		if (this.taskTypeId != null) {
			final TaskSearchQueryV2 searchQuery = new TaskSearchQueryV2();
			searchQuery.addQueryTerm(new String[] { ITaskConstants.TASK_FIELD_TASK_TYPE_ID },
					ITaskSearchQuery.OPERATOR_EQUALS, this.taskTypeId);
			this.taskCount = (int) HPSTRAManager.getInstance().countTasks(searchQuery, false);
		} else {
			this.taskCount = 0;
		}
	}

	public PurgeTasksPortlet getPurgeTasksPortlet() {
		if (this.purgeTasksPortlet == null) {
			this.purgeTasksPortlet = (PurgeTasksPortlet) resolveExpression("#{PurgeTasksPortlet}"); //$NON-NLS-1$
		}
		return this.purgeTasksPortlet;
	}

	public int getTaskCount() {
		return this.taskCount;
	}

	public int getTasksToDelete() {
		return this.tasksToDelete;
	}

	public String getTaskTypeId() {
		return this.taskTypeId;
	}

	public ISelectItemGroupProvider getTaskTypes() {
		return this.taskTypes;
	}

	/**
	 * Initializes page. Loads all task definitions.
	 */
	@Override
	public String initialize() {
		try {
			resolveDataBinding(INITIALIZE_PROPERTY_BINDINGS, null, "initialize", true, false); //$NON-NLS-1$
			final List<SelectItem> list = new ArrayList<>();
			for (final Object taskTypeObj : TaskHelper.getTaskDefinitionService().getAllTaskDefinitions()) {
				final ITaskDefinition taskDef = (ITaskDefinition) taskTypeObj;
				list.add(new SelectItem(taskDef.getTaskTypeID(), taskDef.getTaskDescription()));
			}
			this.taskTypes = new TableSelectItemGroupProvider(list, "label", "value"); //$NON-NLS-1$ //$NON-NLS-2$
			this.taskTypeId = (String) (!list.isEmpty() ? list.get(0).getValue() : null);
			fetchCount();
		} catch (final Exception e) {
			error(e);
			log(e);
		}
		return null;
	}

	public void setTasksToDelete(final int tasksToDelete) {
		this.tasksToDelete = tasksToDelete;
	}

	/**
	 * Synchronizes a given task definition with the HPSTRA storage.
	 * 
	 * @throws Exception
	 *             if any problem occurs
	 */
	public void reIndex() throws Exception {
		long time = System.currentTimeMillis();
		HPSTRAManager.getInstance().updateIndex(TaskHelper.getTaskDefinitionByName(this.taskTypeId));
		time = System.currentTimeMillis() - time;
		Thread.sleep(1000);
		fetchCount();
		String taskDefName = null;
		for (final Object taskTypeObj : TaskHelper.getTaskDefinitionService().getAllTaskDefinitions()) {
			final ITaskDefinition taskDef = (ITaskDefinition) taskTypeObj;
			if (taskDef.getTaskTypeID().equals(this.taskTypeId)) {
				taskDefName = taskDef.getTaskDescription();
				break;
			}
		}
		final String msg = MessageFormat.format(
				"Successfully reindexed task type {0} in {1} seconds in HPSTRA storage", taskDefName, //$NON-NLS-1$
				time / 1000.0);
		getFacesContext().addMessage(null, new FacesMessage(msg));
	}

	public void setTaskTypeId(final String taskTypeId) {
		this.taskTypeId = taskTypeId;
	}

}